@@ -39,10 +39,6 @@
 #include "config.h"
 #endif
 
-#ifdef WIN32
-#include "misc.h"
-#endif
-
 #ifdef DNS_USE_FTIME_FOR_ID
 #include <sys/timeb.h>
 #endif
@@ -1050,7 +1046,7 @@ default_transaction_id_fn(void)
 
 #ifdef DNS_USE_GETTIMEOFDAY_FOR_ID
 	struct timeval tv;
-	gettimeofday(&tv, NULL);
+	evutil_gettimeofday(&tv, NULL);
 	trans_id = tv.tv_usec & 0xffff;
 #endif
 
@@ -1060,7 +1056,7 @@ default_transaction_id_fn(void)
 		/* down to using gettimeofday. */
 		/*
 		  struct timeval tv;
-		  gettimeofday(&tv, NULL);
+		  evutil_gettimeofday(&tv, NULL);
 		  trans_id = tv.tv_usec & 0xffff;
 		*/
 		abort();
