@@ -90,12 +90,57 @@
 #include "http-internal.h"
 
 #ifdef WIN32
-#define snprintf _snprintf
 #define strcasecmp _stricmp
 #define strncasecmp _strnicmp
 #define strdup _strdup
 #endif
 
+#ifndef HAVE_GETNAMEINFO
+#define NI_MAXSERV 32
+#define NI_MAXHOST 1025
+
+#define NI_NUMERICHOST 1
+#define NI_NUMERICSERV 2
+
+int
+fake_getnameinfo(const struct sockaddr *sa, size_t salen, char *host, 
+	size_t hostlen, char *serv, size_t servlen, int flags)
+{
+        struct sockaddr_in *sin = (struct sockaddr_in *)sa;
+        
+        if (serv != NULL) {
+				char tmpserv[16];
+				evutil_snprintf(tmpserv, sizeof(tmpserv),
+					"%d", ntohs(sin->sin_port));
+                if (strlcpy(serv, tmpserv, servlen) >= servlen)
+                        return (-1);
+        }
+
+        if (host != NULL) {
+                if (flags & NI_NUMERICHOST) {
+                        if (strlcpy(host, inet_ntoa(sin->sin_addr),
+                            hostlen) >= hostlen)
+                                return (-1);
+                        else
+                                return (0);
+                } else {
+						struct hostent *hp;
+                        hp = gethostbyaddr((char *)&sin->sin_addr, 
+                            sizeof(struct in_addr), AF_INET);
+                        if (hp == NULL)
+                                return (-2);
+                        
+                        if (strlcpy(host, hp->h_name, hostlen) >= hostlen)
+                                return (-1);
+                        else
+                                return (0);
+                }
+        }
+        return (0);
+}
+
+#endif
+
 #ifndef HAVE_GETADDRINFO
 struct addrinfo {
 	int ai_family;
@@ -152,8 +197,8 @@ fake_freeaddrinfo(struct addrinfo *ai)
 extern int debug;
 
 static int socket_connect(int fd, const char *address, unsigned short port);
-static int bind_socket_ai(struct addrinfo *);
-static int bind_socket(const char *, u_short);
+static int bind_socket_ai(struct addrinfo *, int reuse);
+static int bind_socket(const char *, u_short, int reuse);
 static void name_from_addr(struct sockaddr *, socklen_t, char **, char **);
 static int evhttp_associate_new_request_with_connection(
 	struct evhttp_connection *evcon);
@@ -315,7 +360,7 @@ evhttp_make_header_request(struct evhttp_connection *evcon,
 
 	/* Generate request line */
 	method = evhttp_method(req->type);
-	snprintf(line, sizeof(line), "%s %s HTTP/%d.%d\r\n",
+	evutil_snprintf(line, sizeof(line), "%s %s HTTP/%d.%d\r\n",
 	    method, req->uri, req->major, req->minor);
 	evbuffer_add(evcon->output_buffer, line, strlen(line));
 
@@ -323,7 +368,7 @@ evhttp_make_header_request(struct evhttp_connection *evcon,
 	if (req->type == EVHTTP_REQ_POST &&
 	    evhttp_find_header(req->output_headers, "Content-Length") == NULL){
 		char size[12];
-		snprintf(size, sizeof(size), "%ld",
+		evutil_snprintf(size, sizeof(size), "%ld",
 			 (long)EVBUFFER_LENGTH(req->output_buffer));
 		evhttp_add_header(req->output_headers, "Content-Length", size);
 	}
@@ -380,7 +425,7 @@ evhttp_maybe_add_content_length_header(struct evkeyvalq *headers,
 	if (evhttp_find_header(headers, "Transfer-Encoding") == NULL &&
 	    evhttp_find_header(headers,	"Content-Length") == NULL) {
 		char len[12];
-		snprintf(len, sizeof(len), "%ld", content_length);
+		evutil_snprintf(len, sizeof(len), "%ld", content_length);
 		evhttp_add_header(headers, "Content-Length", len);
 	}
 }
@@ -393,25 +438,35 @@ static void
 evhttp_make_header_response(struct evhttp_connection *evcon,
     struct evhttp_request *req)
 {
+	int is_keepalive = evhttp_is_connection_keepalive(req->input_headers);
 	char line[1024];
-	snprintf(line, sizeof(line), "HTTP/%d.%d %d %s\r\n",
+	evutil_snprintf(line, sizeof(line), "HTTP/%d.%d %d %s\r\n",
 	    req->major, req->minor, req->response_code,
 	    req->response_code_line);
 	evbuffer_add(evcon->output_buffer, line, strlen(line));
 
-	if (req->major == 1 && req->minor == 1)
-		evhttp_maybe_add_date_header(req->output_headers);
+	if (req->major == 1) {
+		if (req->minor == 1)
+			evhttp_maybe_add_date_header(req->output_headers);
 
-	if (req->major == 1 && 
-	    (req->minor == 1 || 
-		evhttp_is_connection_keepalive(req->input_headers))) {
-		/* 
-		 * we need to add the content length if the user did
-		 * not give it, this is required for persistent
-		 * connections to work.
+		/*
+		 * if the protocol is 1.0; and the connection was keep-alive
+		 * we need to add a keep-alive header, too.
 		 */
-		evhttp_maybe_add_content_length_header(req->output_headers,
-		    (long)EVBUFFER_LENGTH(req->output_buffer));
+		if (req->minor == 0 && is_keepalive)
+			evhttp_add_header(req->output_headers,
+			    "Connection", "keep-alive");
+
+		if (req->minor == 1 || is_keepalive) {
+			/* 
+			 * we need to add the content length if the
+			 * user did not give it, this is required for
+			 * persistent connections to work.
+			 */
+			evhttp_maybe_add_content_length_header(
+				req->output_headers,
+				(long)EVBUFFER_LENGTH(req->output_buffer));
+		}
 	}
 
 	/* Potentially add headers for unidentified content. */
@@ -449,7 +504,7 @@ evhttp_make_header(struct evhttp_connection *evcon, struct evhttp_request *req)
 	}
 
 	TAILQ_FOREACH(header, req->output_headers, next) {
-		snprintf(line, sizeof(line), "%s: %s\r\n",
+		evutil_snprintf(line, sizeof(line), "%s: %s\r\n",
 		    header->key, header->value);
 		evbuffer_add(evcon->output_buffer, line, strlen(line));
 	}
@@ -498,7 +553,7 @@ evhttp_hostportfile(char *url, char **phost, u_short *pport, char **pfile)
 		/* Generate request file */
 		if (p2 == NULL)
 			p2 = "";
-		snprintf(file, sizeof(file), "/%s", p2);
+		evutil_snprintf(file, sizeof(file), "/%s", p2);
 	}
 
 	p = strchr(host, ':');
@@ -1531,7 +1586,7 @@ evhttp_connection_connect(struct evhttp_connection *evcon)
 	assert(!(evcon->flags & EVHTTP_CON_INCOMING));
 	evcon->flags |= EVHTTP_CON_OUTGOING;
 	
-	evcon->fd = bind_socket(evcon->bind_address, 0);
+	evcon->fd = bind_socket(evcon->bind_address, 0 /*port*/, 0 /*reuse*/);
 	if (evcon->fd == -1) {
 		event_debug(("%s: failed to bind to \"%s\"",
 			__func__, evcon->bind_address));
@@ -2002,7 +2057,7 @@ evhttp_bind_socket(struct evhttp *http, const char *address, u_short port)
 	int fd;
 	int res;
 
-	if ((fd = bind_socket(address, port)) == -1)
+	if ((fd = bind_socket(address, port, 1 /*reuse*/)) == -1)
 		return (-1);
 
 	if (listen(fd, 128) == -1) {
@@ -2282,7 +2337,7 @@ evhttp_get_request_connection(
 	int fd, struct sockaddr *sa, socklen_t salen)
 {
 	struct evhttp_connection *evcon;
-	char *hostname, *portname;
+	char *hostname = NULL, *portname = NULL;
 
 	name_from_addr(sa, salen, &hostname, &portname);
 	event_debug(("%s: new request from %s:%s on %d\n",
@@ -2388,32 +2443,37 @@ static void
 name_from_addr(struct sockaddr *sa, socklen_t salen,
     char **phost, char **pport)
 {
-#ifdef HAVE_GETNAMEINFO
-	/* XXXX not threadsafe. */
-	static char ntop[NI_MAXHOST];
-	static char strport[NI_MAXSERV];
+	char ntop[NI_MAXHOST];
+	char strport[NI_MAXSERV];
 	int ni_result;
 
-	if ((ni_result = getnameinfo(sa, salen,
+#ifdef HAVE_GETNAMEINFO
+	ni_result = getnameinfo(sa, salen,
 		ntop, sizeof(ntop), strport, sizeof(strport),
-		NI_NUMERICHOST|NI_NUMERICSERV)) != 0) {
+		NI_NUMERICHOST|NI_NUMERICSERV);
+	
+	if (ni_result != 0) {
 		if (ni_result == EAI_SYSTEM)
 			event_err(1, "getnameinfo failed");
 		else
 			event_errx(1, "getnameinfo failed: %s", gai_strerror(ni_result));
+		return;
 	}
- 
+#else
+	ni_result = fake_getnameinfo(sa, salen,
+		ntop, sizeof(ntop), strport, sizeof(strport),
+		NI_NUMERICHOST|NI_NUMERICSERV);
+	if (ni_result != 0)
+			return;
+#endif
 	*phost = ntop;
 	*pport = strport;
-#else
-	/* XXXX */
-#endif
 }
 
 /* Either connect or bind */
 
 static int
-bind_socket_ai(struct addrinfo *ai)
+bind_socket_ai(struct addrinfo *ai, int reuse)
 {
         int fd, on = 1, r;
 	int serrno;
@@ -2436,7 +2496,10 @@ bind_socket_ai(struct addrinfo *ai)
 #endif
 
         setsockopt(fd, SOL_SOCKET, SO_KEEPALIVE, (void *)&on, sizeof(on));
-        setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (void *)&on, sizeof(on));
+	if (reuse) {
+		setsockopt(fd, SOL_SOCKET, SO_REUSEADDR,
+		    (void *)&on, sizeof(on));
+	}
 
 	r = bind(fd, ai->ai_addr, ai->ai_addrlen);
 	if (r == -1)
@@ -2465,7 +2528,7 @@ make_addrinfo(const char *address, u_short port)
         ai.ai_family = AF_INET;
         ai.ai_socktype = SOCK_STREAM;
         ai.ai_flags = AI_PASSIVE;  /* turn NULL host name into INADDR_ANY */
-        snprintf(strport, sizeof(strport), "%d", port);
+        evutil_snprintf(strport, sizeof(strport), "%d", port);
         if ((ai_result = getaddrinfo(address, strport, &ai, &aitop)) != 0) {
                 if ( ai_result == EAI_SYSTEM )
                         event_warn("getaddrinfo");
@@ -2490,7 +2553,7 @@ make_addrinfo(const char *address, u_short port)
 }
 
 static int
-bind_socket(const char *address, u_short port)
+bind_socket(const char *address, u_short port, int reuse)
 {
 	int fd;
 	struct addrinfo *aitop = make_addrinfo(address, port);
@@ -2498,7 +2561,7 @@ bind_socket(const char *address, u_short port)
 	if (aitop == NULL)
 		return (-1);
 
-	fd = bind_socket_ai(aitop);
+	fd = bind_socket_ai(aitop, reuse);
 
 #ifdef HAVE_GETADDRINFO
 	freeaddrinfo(aitop);
